pro ng_make_dullness_matrix,response_matrix,d_response_matrix, $
                            dullness_matrix,d_dullness_matrix, $
                            DIAGONAL=diagonal

; S. Baba (20160908)
; This procedure invert a response matrix and its uncertainty analytically.
;
; inputs
;   response_matrix: a matrix of responses
;   d_responses_matrix: a matrix of uncertainties
; outputs
;   dullness_matrix: the invert matrix of the responpose matrix
;                    calculated from response_matrix analytically
;   d_dullness_matrix: uncertainties of the dullness matrix
;                    calculated from d_response_matrix analytically with
;                    the error propagation law.
; option
;   DIAGONAL: If this keyword is set, off-diagonal elemtnts of
;             the dullness matrix is flux calibration is set to 0
;             and thus the 2nd-order response is ignored.
;

  ;dullness_matrix = invert(response_matrix)
  
   r =   response_matrix
  dr = d_response_matrix
  
  sz = size(r)
  if sz[0] ne 2 then message, 'Response matrix must has two dimensions.'
  if sz[1] ne sz[2] then message, 'Response matrix must be square.'
  n = sz[1]
  
    dullness_matrix = make_array(n,n,/DOUBLE,VALUE=0.)
  d_dullness_matrix = make_array(n,n,/DOUBLE,VALUE=0.)
  
  nonzero_1D = where(r GT 0.0D)
  nonzero_2D = array_indices(r,nonzero_1D)
  
  ;print, '                  calculating inverse response matrix (=dullness matrix)...'
  for l=0, n_elements(nonzero_1D)-1 do begin
    i = nonzero_2D[0,l]
    j = nonzero_2D[1,l]
    if i eq j then begin
      ;diagonal elements
        dullness_matrix[i,i] = 1.0D / r[i,i]
      d_dullness_matrix[i,i] = dr[i,i] / (r^2)[i,i]
    endif else begin
      if ~keyword_set(DIAGONAL) then begin
        ;off-diagonal elements
          dullness_matrix[i,j] = - r[i,j] / r[i,i] / r[j,j]
        d_dullness_matrix[i,j] = dullness_matrix[i,j] * sqrt( ((dr/r)^2)[i,j] + ((dr/r)^2)[i,i] + ((dr/r)^2)[j,j] )
      endif
    endelse
  endfor

end
