pro ng_make_response_matrix,lambda_1st,              lambda_2nd, $
                            response_at_lambda_1st,  response_at_lambda_2nd, $
                            d_response_at_lambda_1st,d_response_at_lambda_2nd, $
                            response_matrix, $
                            d_response_matrix, $
                            DIAGONAL=diagonal

; S. Baba (20160908)
; This procedure makes a response matrix with its uncertainty from
; spectral responces functions of 1st- and 2nd-order light.
; 
; inputs
;   lambda: a vector of wavelengths
;   response: a vector of responces at lambda
;   d_response: a vector of uncertainties
; outpus
;   response_matrix: a matrix of responses
;   d_responses_matrix: a matrix of uncertainties
;   See Baba et al. (2016) for the definition.
; option
;   DIAGONAL: If this keyword is set, off-diagonal elemtnts of 
;             the response matrix is flux calibration is set to 0
;             and thus the 2nd-order response is ignored.
;

  ; abbreviation
   l1 = lambda_1st
   r1 =   response_at_lambda_1st
  dr1 = d_response_at_lambda_1st  
   l2 = lambda_2nd
   r2 =   response_at_lambda_2nd
  dr2 = d_response_at_lambda_2nd
  
  ; check if lambda_1st has descending order.
  l1_des = l1[reverse(sort(l1))]
  if ~array_equal(l1,l1_des) then message, 'lambda_1st must has descending order.'
  
  ; size check
  n = n_elements(l1)
  if ~(   n EQ n_elements( l2) $
       && n EQ n_elements( r1) $
       && n EQ n_elements( r2) $
       && n EQ n_elements(dr1) $
       && n EQ n_elements(dr2)  ) then message, 'Size mismatch.'
  
    response_matrix = make_array(n,n,/DOUBLE,VALUE=0.)
  d_response_matrix = make_array(n,n,/DOUBLE,VALUE=0.)
  
  ; diagonal elements
  ;print, '                  substituting response_1st for diagonal elements of the response matrix...'
  for i=0, n-1 do begin
      response_matrix[i,i] =  r1[i]
    d_response_matrix[i,i] = dr1[i]
  endfor
  
  if ~keyword_set(DIAGONAL) then begin
    ; off-diagonal elements
    ;print, '                  substituting response_2nd for off-diagonal elements of the response matrix...'
    linterp, l2, indgen(n-1), l1[n-1], id_overlap
    k = floor(id_overlap)
    for j=0, k do begin
      neighbor_l1_id = neighbor_id(l1, l2[j])
      id_l = neighbor_l1_id[0]
      id_s = neighbor_l1_id[1]
      l1_l = l1[id_l]
      l1_s = l1[id_s]
        response_matrix[id_s,j] =  r2[j] * ( l1_l - l2[j] ) / ( l1_l - l1_s )
        response_matrix[id_l,j] =  r2[j] * ( l2[j] - l1_s ) / ( l1_l - l1_s )
      d_response_matrix[id_s,j] = dr2[j] * ( l1_l - l2[j] ) / ( l1_l - l1_s )
      d_response_matrix[id_l,j] = dr2[j] * ( l2[j] - l1_s ) / ( l1_l - l1_s )
    endfor
  endif

end
