function ng_waveget, pos_array

; S. Baba (20160908)
; This function converts positions along the dispersion direction to 
; 1st-order wavelengths with a wavelength calibration curve obtained 
; by considering the wavelength dependence of refractive index of germanium.
; The wavelength calibration curve is shown in Fig. 5 of Baba et al. (2016).
; Note that Baba et al. (2016) take the sign of the positions so that 
; positive positions correspond to longer wavelengths.
; Its direction is opposite to the direction adopted in this toolkit.
; 
; input
;   pos_array: a vector of positions on the detector
; output
;   wave_array: a vector of 1st-order wavelengths
  
  ; read the wavelength calibration curve
  set_wavetable_ng,pos_tab,lambda_1st,lambda_2nd
  
  ; linear interpolation
  linterp,pos_tab,lambda_1st,pos_array,wave_array,MISSING=0.
  
  return, wave_array
  
end
